/*
* JBoss, Home of Professional Open Source
* Copyright 2006, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.managed.plugins.factory;

import org.jboss.managed.spi.factory.RuntimeComponentNameTransformer;

/**
 * Abstract component name transformer.
 *
 * @param <T> exact value type
 * @author <a href="mailto:ales.justin@jboss.com">Ales Justin</a>
 */
public abstract class AbstractComponentNameTransformer<T> implements RuntimeComponentNameTransformer
{
   private Class<T> type;

   protected AbstractComponentNameTransformer(Class<T> type)
   {
      if (type == null)
         throw new IllegalArgumentException("Null type.");
      this.type = type;
   }

   public Object transform(Object value)
   {
      if (value == null)
         return null;

      if (type.isAssignableFrom(value.getClass()) == false)
         return new IllegalArgumentException("Value is not instance of " + type + ", value: " + value);

      return doTransform(type.cast(value));
   }

   /**
    * Transform on exact type.
    *
    * @param value the value
    * @return transformed name
    */
   protected abstract Object doTransform(T value);
}
